﻿using System;
using Windows.ApplicationModel.DataTransfer;
using Windows.Media.SpeechRecognition;
using Windows.Media.SpeechSynthesis;
using Windows.Phone.UI.Input;
using Windows.Storage;
using Windows.Storage.Streams;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Navigation;
using FirstWP81App.ViewModels;

namespace FirstWP81App
{
	/// <summary>
	/// An empty page that can be used on its own or navigated to within a Frame.
	/// </summary>
	public sealed partial class DetailsPage : Page
	{
		private DataTransferManager _dataTransferManager;
		private ItemViewModel _selectedItem;

		public DetailsPage()
		{
			this.InitializeComponent();
			this.NavigationCacheMode = NavigationCacheMode.Required;
			HardwareButtons.BackPressed += HardwareButtons_BackPressed;
		}

		protected override void OnNavigatedTo(NavigationEventArgs e)
		{
			int index = int.Parse(e.Parameter.ToString());
			_selectedItem = App.ViewModel.Items[index];
			DataContext = _selectedItem;

			_dataTransferManager = DataTransferManager.GetForCurrentView();
			_dataTransferManager.DataRequested += _dataTransferManager_DataRequested;
		}

		protected override void OnNavigatedFrom(NavigationEventArgs e)
		{
			_dataTransferManager.DataRequested -= _dataTransferManager_DataRequested;
			base.OnNavigatedFrom(e);
		}

		private void HardwareButtons_BackPressed(object sender, BackPressedEventArgs e)
		{
			Frame.GoBack();
			e.Handled = true;
		}

		async void _dataTransferManager_DataRequested(DataTransferManager sender, DataRequestedEventArgs args)
		{
			var deferral = args.Request.GetDeferral();

			try
			{
				DataPackage requestData = args.Request.Data;
				requestData.Properties.Title = _selectedItem.Title;
				requestData.Properties.Description = _selectedItem.Author;

				var folder = ApplicationData.Current.LocalFolder;
				var photoFile = await folder.CreateFileAsync("photo.jpg", CreationCollisionOption.ReplaceExisting);
				var downloader = new Windows.Networking.BackgroundTransfer.BackgroundDownloader();
				var dl = downloader.CreateDownload(_selectedItem.ImageUri, photoFile);
				await dl.StartAsync();
		
				RandomAccessStreamReference streamRef = RandomAccessStreamReference.CreateFromUri(_selectedItem.ImageUri);
				requestData.Properties.Thumbnail = streamRef;
				requestData.SetBitmap(streamRef);
				requestData.SetWebLink(_selectedItem.ImageUri);
				requestData.SetStorageItems(new[] { photoFile });
			}
			finally
			{
				deferral.Complete();
			}
		}

		private void Share_Click(object sender, RoutedEventArgs e)
		{
			DataTransferManager.ShowShareUI();
		}

		private async void Read_Click(object sender, RoutedEventArgs e)
		{
			SpeechSynthesizer ss = new SpeechSynthesizer();
			ss.Voice = SpeechSynthesizer.AllVoices[1];
			MediaElement me = new MediaElement();
			SpeechSynthesisStream stream = await ss.SynthesizeTextToStreamAsync(_selectedItem.Title);
			me.SetSource(stream, stream.ContentType);
			me.Play();
		}
	}
}
